﻿define pong_zoom = 8

init:
    image pong_ball:
        Crop((0,0,6,6), "free pong/sprite.png")
        zoom pong_zoom nearest True
    image pong_paddle:
        Crop((16,0,4,32), "free pong/sprite.png")
        zoom pong_zoom nearest True

init python:
    import pygame
    class pong_ball(renpy.Displayable):
        def __init__(self, image):
            super(renpy.Displayable,self).__init__()
            self.image = image
            self.x = 0
            self.y = 0

            self.visible = True
            self.parent = None
            self.colliding = 0

            self.x_direction = random.choice([-1,1])
            self.y_direction = random.choice([-1,1])

            self.x_velocity = random.randint(4,6)
            self.y_velocity = random.randint(4,6)
        def collision_check(self):
            if self.colliding > 0:
                self.colliding -= 1
                return
            for i in self.parent.paddles:
                if distance(self.x, i.x) < 30:
                    if distance(self.y, i.y) < 128:
                        self.colliding = 50
                        self.x_direction = -self.x_direction
                        self.x_velocity += 1
                        self.y_velocity += i.y_velocity//2
                        renpy.play("free pong/hit.ogg", "sound")
                else:
                    self.colliding = False
        def render(self,width,height,st,at):
            if not self.visible:
                return renpy.Render(0,0)
            
            self.x += self.x_direction * self.x_velocity
            self.y += self.y_velocity

            if abs(self.x) > 1000:
                self.visible = False
                self.parent.goal(self.x)



            # if self.x < -900:
            #     if self.x_direction < 0:
            #         self.x_direction = -self.x_direction
            # elif self.x > 900:
            #     if self.x_direction > 0:
            #         self.x_direction = -self.x_direction
            if self.y < -500:
                if self.y_velocity < 0:
                    self.y_velocity = -self.y_velocity
            elif self.y > 500:
                if self.y_velocity > 0:
                    self.y_velocity = -self.y_velocity

            self.collision_check()

            t = Transform(self.image)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (self.x, self.y))

            renpy.redraw(self, 0)
            return rv
    class pong_paddle(renpy.Displayable):
        def __init__(self, image, x, is_ai = False):
            super(renpy.Displayable,self).__init__()
            self.image = image
            self.x = x
            self.y = 0

            self.visible = True
            self.parent = None
            self.is_ai = is_ai
            self.score = 0

            self.w = 0
            self.a = 0
            self.s = 0
            self.d = 0

            self.speed = 2
            self.x_velocity = 0
            self.y_velocity = 0

        def render(self,width,height,st,at):
            if not self.visible:
                return renpy.Render(0,0)
            
            if self.is_ai:
                ball = self.parent.balls[0]
                if ball.y > self.y:
                    self.y_velocity += 1*self.speed
                else:
                    self.y_velocity -= 1*self.speed
                self.y_velocity += (self.w + self.s)*self.speed
            else:
                self.y_velocity += (self.w + self.s)*self.speed

            if self.y_velocity > 0:
                self.y_velocity -= 1
            elif self.y_velocity < 0:
                self.y_velocity += 1
            
            self.y += self.y_velocity

            if self.y < -400:
                self.y = -400
                self.y_velocity = 0
            elif self.y > 400:
                self.y = 400
                self.y_velocity = 0

            t = Transform(self.image)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (self.x, self.y))

            renpy.redraw(self, 0)
            return rv

        def event(self, ev, x, y, st):
            if self.is_ai:
                return
            if ev.type == pygame.KEYDOWN:
                if ev.key == pygame.K_w:
                    self.w = -1
                    renpy.redraw(self, 0)
                    raise renpy.IgnoreEvent()
                elif ev.key == pygame.K_s:
                    self.s = 1
                    renpy.redraw(self, 0)
                    raise renpy.IgnoreEvent()
            elif ev.type == pygame.KEYUP:
                if ev.key == pygame.K_w:
                    self.w = 0
                elif ev.key == pygame.K_s:
                    self.s = 0
            return None

    class pong_class:
        def __init__(self, paddles):
            self.paddles = paddles
            for i in paddles:
                i.parent = self
            self.balls = []
            self.add_ball()
            self.stat = "playing"
        def add_ball(self, rest = True):
            if reset:
                self.balls = []
            ball = pong_ball("pong_ball")
            ball.parent = self
            self.balls.append(ball)
            self.stat = "playing"
        def goal(self, x):
            if x > 0:
                for i in self.paddles:
                    if i.x < 0:
                        winner = i
            else:
                for i in self.paddles:
                    if i.x > 0:
                        winner = i
            winner.score += 1
            if winner.is_ai:
                renpy.play("free pong/wrong.ogg", "sound")
            else:
                renpy.play("free pong/win.ogg", "sound")
            self.stat = "waiting"
            renpy.restart_interaction()
        def multiply_balls(self):
            if len(self.balls) > 4:
                return
            balls = []
            for i in self.balls:
                ball = pong_ball("pong_ball")
                ball.parent = self
                ball.x = i.x
                ball.y = i.y
                balls.append(ball)
            self.balls += balls


screen pong_screen():
    default g = pong_class(
        [
            pong_paddle("pong_paddle", 900),
            pong_paddle("pong_paddle", -900, True),
        ]
    )
    add "#e0cfff"
    frame:
        align .5,.5 xysize 1920,1040 background "#04000a"

    for i in g.paddles:
        add i align .5,.5
        text str(i.score) align .5,.5 xoffset i.x//2 size 200 color "#160f1f"
    for i in g.balls:
        add i align .5,.5

    if g.stat == "waiting":
        button:
            align .5,.5 background "#160f1f" padding 20,20
            text "Throw"
            action Function(g.add_ball)


    key "`" action Function(g.multiply_balls)


label pong_example:
    menu:
        "Classic pong":
            call screen pong_screen
        "Free pong":
            call screen free_pong_screen